import {NaiveUiResolver} from "unplugin-vue-components/dist/resolvers.js";

import {nodes, is_mac, zip_file} from "../tool/nodes.js";

import viteCompression from "vite-plugin-compression";

import components from 'unplugin-vue-components/vite';

import autoImport from "unplugin-auto-import/vite";

import {createHtmlPlugin} from "vite-plugin-html";

import markdown from 'unplugin-vue-markdown/vite';

import {viteMockServe} from "vite-plugin-mock";

import {delView} from "../route/setVue.js";

import svgLoader from "vite-svg-loader";

import frameConfig from "./config.js";

import vue from "@vitejs/plugin-vue";

import alone from "../tool/alone.js";

import {defineConfig} from "vite";

const setPlugin = function (list) {
    return Object.keys(list).reduce((acc, key) => {
        acc[key] = function (data) {
            return {plugin_name: key, ['plugin_' + key]: data};
        };
        return acc;
    }, {});
};

const callPlugin = function (list) {
    return Object.values(list).map(method => method());
};

const callPlugins = async function (list) {
    return await Promise.all(callPlugin(list));
};

const framePlugin = function (then) {
    return {
        vue(data) {
            return vue(alone.array_merge({include: [/\.vue$/, /\.md$/]}, data));
        },
        html(data) {
            const tags = alone.get_arr(data, 'inject.tags');
            const config = alone.array_merge(data, {minify: true, inject: {tags: []}});
            config.inject.tags = (alone.isset(tags) && alone.is_array(tags)) ? then.html_tag.concat(tags) : then.html_tag;
            config.entry = then.option.main;
            return createHtmlPlugin(config);
        },
        auto(data) {
            return autoImport(alone.array_merge({
                dts: then.env.dts.auto || false,
                imports: ['vue', {
                    'naive-ui': [
                        'useDialog',
                        'useMessage',
                        'useNotification',
                        'useLoadingBar'
                    ]
                }]
            }, data || {}));
        },
        component(data) {
            return components(alone.array_merge({
                dts: then.env.dts.component || false,
                extensions: ['vue', 'md'],
                include: [/\.vue$/, /\.vue\?vue/, /\.md$/],
                resolvers: [NaiveUiResolver()],
            }, data || {}));
        },
        svg(data) {
            return svgLoader(alone.array_merge({defaultImport: 'url'}, data));
        },
        md(data) {
            then.deploy.markdown = true;
            const config = {headEnabled: true, markdownItOptions: {html: true, linkify: true, typographer: true}};
            const include = [`${then.node.get_path('.')}/*.md`, `${then.node.get_path('.')}/**/*.md`];
            const includes = alone.get_arr(data, 'include');
            config.include = (alone.isset(includes) && alone.is_array(includes)) ? include.concat(includes) : include;
            return markdown(config);
        },
        mock(data) {
            if (!then.env.mock) return {};
            return viteMockServe(alone.array_merge({
                mockPath: then.env.mock,//存放 Mock 文件的路径。默认情况下，插件会在此路径下查找 Mock 文件
                watchFiles: true,//指示是否监视 Mock 文件的变化。如果设置为 true，当 Mock 文件发生变化时，Mock 服务器会自动更新。
                enable: true,//指示是否启用 Mock 功能。设置为 false 可以临时禁用 Mock。
                logger: false,//指示是否启用插件的日志记录。设置为 true 会在终端输出 Mock 请求的日志信息。
                cors: true //指示是否启用 CORS（跨域资源共享）。设置为 true 会在响应中添加 CORS 头部，以允许跨域请求。
            }, data || {}));
        },
        compress(data) {
            return viteCompression(alone.array_merge({
                verbose: true,    // 默认即可
                disable: true,  //开启压缩(不禁用)，默认即可
                deleteOriginFile: false, //删除源文件
                threshold: 10240, //压缩前最小文件大小
                algorithm: 'gzip',  //压缩算法
                ext: '.gz', //文件类型
            }, data));
        }
    };
};

const alonePlugin = function (then) {
    return {
        //设置配置
        alone_config() {
            return {
                enforce: 'pre',
                name: 'alone-config',
                config(config) {
                    config.root = then.app_path;
                    config.base = config.base || "./";
                    config.publicDir = then.publicDir();
                    config.resolve = config.resolve || {};
                    config.define = alone.array_merge(config.define, then.define());
                    config.resolve.alias = alone.array_merge(config.resolve.alias, then.alias());
                    config.build = alone.array_merge(config.build, then.plugin_build());
                    config.preview = alone.array_merge(config.preview, then.preview());
                    config.server = alone.array_merge(config.server, then.server());
                },
                buildStart() {
                    delView();
                    if (then.update) {
                        then.is_del_vite = true;
                        then.node.deletes_file('node_modules/.vite');
                    }
                }
            };
        },
        //加载main
        alone_main() {
            return {
                enforce: 'pre',
                name: 'alone-main',
                transform(code, file) {
                    const beforeCode = `
            import MainLoadBefore from "alone-frame/frame/main";
            MainLoadBefore.status = {};
            await MainLoadBefore.load();
            `;
                    const afterCode = `
            import MainLoadAfter from "alone-frame/frame/main";
            MainLoadAfter.mount();
            `;
                    return then.plugin_transform_main(code, file, beforeCode, afterCode);
                }
            };
        },
        //显示
        alone_show() {
            return {
                enforce: 'post',
                name: 'alone-show',
                buildStart() {
                    console.log('path', `${is_mac() ? "file://" : ""}${then.app.get_path('.')}`);
                },
                closeBundle() {
                    console.log('complete', `${is_mac() ? "file://" : ""}` + then.out_dir);
                }
            };
        },
        //路由
        alone_route() {
            return {
                enforce: 'pre',
                name: 'alone-route',
                buildStart() {
                    then.vue_route();
                }
            };
        },
        alone_build() {
            return {
                enforce: 'post',
                apply: 'build',
                name: 'alone-build',
                buildStart() {
                    if (!then.is_del_vite) {
                        then.node.deletes_file('node_modules/.vite');
                    }
                    nodes(then.out_dir).del_dir_file('.', then.env.exclude);
                },
                closeBundle() {
                    nodes(then.out_dir).delete_file('layui/lay.js');
                    if (then.env.zip === true) {
                        zip_file(then.out_dir, alone.rtrim(then.out_dir, '/') + ".zip", function (size, unit, file) {
                            console.log('zip file', file);
                            console.log('zip size', `${size}${unit}`);
                        });
                    }
                }
            };
        },
        alone_file_reload() {
            return {
                name: 'alone-file-reload',
                handleHotUpdate({file, server}) {
                    const files = then.trim_rep(file);
                    const include = [
                        then.trim_rep(then.app.get_path(then.option.conf)),
                        then.trim_rep(then.app.get_path(then.option.env)),
                        then.trim_rep(then.app.get_path(then.option.vite))
                    ];
                    if (include.includes(files)) {
                        console.log(`\x1b[90m${new Date().toLocaleTimeString()}\x1b[0m [alone] \x1b[32mpage reload\x1b[0m ${files.split('/').pop()}`);
                        server.ws.send({type: 'full-reload'});
                        //server.restart();
                    }
                },
            }
        }
    }
};

const viteConfig = async function (then, pluginList, pluginSet, viteRes) {
    let list_plugin = {}, load_plugin = {}, plugins = [];
    if (alone.isset(viteRes)) {
        if (alone.is_array(viteRes.plugins) && alone.isset(viteRes.plugins)) {
            for (const [key, val] of Object.entries(viteRes.plugins)) {
                const name = val.plugin_name;
                if (name && pluginList.hasOwnProperty(name)) {
                    list_plugin[name] = val['plugin_' + name];
                    if (list_plugin[name] !== false) {
                        load_plugin[name] = await (pluginList[name])(list_plugin[name]);
                    }
                } else {
                    load_plugin[key] = val;
                }
            }
            if (alone.isset(list_plugin)) {
                Object.keys(pluginList).forEach((key) => {
                    if (!list_plugin.hasOwnProperty(key)) {
                        plugins.push(pluginList[key]());
                    }
                });
            } else {
                plugins = plugins.concat(await callPlugins(pluginList));
            }
            Object.keys(load_plugin).forEach((key) => plugins.push(load_plugin[key]));
        }
    }
    viteRes.plugins = (alone.isset(plugins) ? plugins : await callPlugins(pluginList));
    return viteRes;
}

export default function (pathname = 'src', callback = null) {
    return defineConfig(async function (option) {
        const then = await frameConfig(pathname || 'src');
        const pluginList = Object.assign(framePlugin(then), alonePlugin(then));
        const pluginSet = setPlugin(pluginList);
        const call = alone.is_callback(callback) ? callback : then.conf_vite;
        const viteRes = alone.is_callback(call) ? (await call(option, then, pluginSet)) : {};
        return await viteConfig(then, pluginList, pluginSet, viteRes);
    });
}